from . import pna_tools
import numpy as np
import warnings


def checkWarnings(seq):
    """

    :param seq:
    :return:
    """
    sequence_warnings = []
    if len(seq) > 30:
        sequence_warnings.append('PNA sequence too long')
    if partialComp(seq, 7):
        sequence_warnings.append('Significant self-complementary sequence (>6 bp)')

    purine_check_1 = purine_check_2 = g_check = False

    if len(seq) > 10:
        for ii in range(0, len(seq) - 9):
            sub_seq = seq[ii:ii + 10]
            if 'PPPPP' in purAndPyr(sub_seq):
                purine_check_1 = True
            if purAndPyr(sub_seq).count('P') > 6:
                purine_check_2 = True
            if 'GGGG' in purAndPyr(sub_seq):
                g_check = True
    else:
        if 'PPPPP' in purAndPyr(seq):
            purine_check_1 = True
        if purAndPyr(seq).count('P') > 6:
            purine_check_2 = True
        if 'GGGG' in purAndPyr(seq):
            g_check = True

    if purine_check_1:
        sequence_warnings.append('Purine stretch of 5 or greater')
    if purine_check_2:
        sequence_warnings.append('High purine content')
    if g_check:
        sequence_warnings.append('Guanine stretch of 4 or greater')

    return sequence_warnings


def purAndPyr(sequence):
    """

    :param sequence:
    :return:
    """
    if type(sequence) is not str:
        raise TypeError('purAndPyr takes a sequence string')

    sequence = sequence.upper()
    type_seq = ''

    for letter in sequence:
        if letter == 'A' or letter == 'G':
            type_seq += 'P'
        elif letter == 'T' or letter == 'U' or letter == 'C':
            type_seq += 'Y'
        else:
            raise ValueError('Non-RNA or non-DNA base included')

    return type_seq


def partialComp(seq, comp_len):
    """

    :param seq:
    :param comp_len:
    :return:
    """
    if type(seq) is not str:
        raise TypeError('partialComp takes a sequence string')

    seq = seq.upper()
    slide = len(seq) - comp_len + 1

    for ii in range(slide):
        subseq = seq[ii:ii + comp_len]
        rcompseq = pna_tools.revComp(subseq)
        compseq = rcompseq[::-1]

        if rcompseq in seq or compseq in seq:
            return True

    return False


def calculateTmPNA(sequence, ct=1e-4, R=1.987e-3, nacl=1, dH_dict=None, dS_dict=None, dG_dict=None):
    """

    :param sequence: N to C PNA sequence
    :param ct:
    :return:
    """
    try:
        sequence = sequence.upper()
        for letter in sequence:
            if letter not in 'ACGT':
                if letter == 'U':
                    raise ValueError('RNA base included. calculateTmPNA takes a DNA sequence string')
                else:
                    raise ValueError('calculateTmPNA takes a DNA sequence string')
    except AttributeError:
        raise TypeError('calculateTmPNA takes a string')

    if R != 1.987e-3:
        warnings.warn('Gas constant value altered, be sure to use appropriate nearest neighbor parameters')

    if not dH_dict:
        dH_dict = {'AA': -8.4, 'AT': -6.5, 'TA': -6.3, 'CA': -7.4, 'GT': -8.6, 'CT': -6.1, 'GA': -7.7, 'CG': -10.1,
                   'GC': -11.1, 'GG': -6.7}

    if not dS_dict:
        dS_dict = {'AA': -23.6, 'AT': -18.8, 'TA': -18.5, 'CA': -19.3, 'GT': -23, 'CT': -16.1, 'GA': -20.3, 'CG': -25.5,
                   'GC': -28.4, 'GG': -15.6}

    if not dG_dict:
        dG_dict = {'AA': -1.02, 'AT': -0.73, 'TA': -0.6, 'CA': -1.38, 'GT': -1.43, 'CT': -1.16, 'GA': -1.46,
                   'CG': -2.09, 'GC': -2.28, 'GG': -1.77}

    dH = dS = dG = 0

    if 'G' in sequence or 'C' in sequence:
        dS += -5.9
        dG += 1.82
    else:
        dS += -9.0
        dG += 2.8

    if sequence == pna_tools.revComp(sequence):
        dS += -1.4
        dG += 0.4
    else:
        ct *= 0.25

    if sequence[-1] == 'T':
        dH += 0.4
        dG += 0.4

    for ii in range(len(sequence) - 1):
        pair = sequence[ii:ii + 2]

        try:
            dH += dH_dict[pair]
        except KeyError:
            dH += dH_dict[pna_tools.revComp(pair)]

        try:
            dS += dS_dict[pair]
        except KeyError:
            dS += dS_dict[pna_tools.revComp(pair)]

        try:
            dG += dG_dict[pair]
        except KeyError:
            dG += dG_dict[pna_tools.revComp(pair)]

    dS *= 0.001
    temp_dna = dH / (dS + R * np.log(ct)) - 273.15
    tm_dna_salt = temp_dna + 12.5 * np.log(nacl)
    c = (20.79, 0.83, -26.13, 0.44)
    temp_pna = c[0] + (c[1] * temp_dna) + (c[2] * purAndPyr(sequence).count('Y') / len(sequence)) + (c[3] * len(sequence))

    return dH, dS, dG, temp_dna, temp_pna



# Initial setup module for the PNA Finder toolbox

from tkinter import *
import tkinter.filedialog
import functools
import os


class start_app:
    def __init__(self, master):
        """
        Initializes the PNA Finder start_info app, in order to set home and bash directories.
        (see manual.pdf in package root directory for more info)
        :param master: tkinter GUI frame
        """
        self.master = master
        self.master.title('Choose toolbox startup settings')

        # Dictionaries for filenames and GUI label
        self.labels = {}
        self.flabels = {}
        self.files = {}

        # Variable initialization
        self.submit_warning = None

        # Header
        self.labels['header'] = Label(
            self.master,
            text=15*' ' + 'PNA Finder Startup Settings' + 15*' ',
            font=('Arial', 11, 'bold')
        )
        self.labels['header'].grid(row=0, columnspan=3)

        # Home directory
        self.labels['home'] = Label(
            self.master,
            font=('Arial', 9, 'bold'),
            text='Home directory'
        )
        self.labels['home'].grid(row=1, column=0, sticky=W, padx=5, pady=5)

        home_button = Button(
            self.master,
            text="Select",
            command=functools.partial(self.browse_folder, self.master, 'home',
                                      title='Select home directory',
                                      row=1, column=1))
        home_button.grid(row=1, column=2, sticky=E, padx=5, pady=5)

        # Bash directory
        self.labels['bash'] = Label(
            self.master,
            text='Bash shell directory',
            font=('Arial', 9, 'bold'),
        )
        self.labels['bash'].grid(row=2, column=0, sticky=W, padx=5, pady=5)

        bash_button = Button(
            self.master,
            text="Select",
            command=functools.partial(self.browse_folder, self.master, 'bash',
                                      title='Select Bash shell directory',
                                      row=2, column=1))
        bash_button.grid(row=2, column=2, sticky=E, padx=5, pady=5)

        # Bash shell type selection
        Label(
            self.master,
            text='Bash shell type:', font=("Arial", 9, "bold")
        ).grid(row=3, column=0, sticky=W, padx=5, pady=5)

        self.bash_option = IntVar()
        self.bash_option.set(0)

        Radiobutton(
            self.master,
            text='Cygwin',
            variable=self.bash_option, value=0
        ).grid(row=3, column=1, sticky=W, padx=5, pady=5)

        Radiobutton(
            self.master,
            text='Windows Bash',
            variable=self.bash_option, value=1
        ).grid(row=3, column=2, sticky=W, padx=5, pady=5)

        # OK and QUIT buttons
        Button(
            self.master,
            text="OK", command=self.writeStartInfo
        ).grid(row=10, column=0, sticky=W, padx=5, pady=10)

        Button(
            self.master,
            text="QUIT", fg="red", command=self.master.quit,
        ).grid(row=10, column=2, sticky=E, padx=5, pady=10)

    def browse_folder(self, frame, filekey, title='Select folder', row=0, column=0):
        """
        Function prompts user to select a folder for input into the start_info dialog, and records choice in GUI display
        :param frame: Tkinter active frame
        :param filekey: Key for the self.files dictionary, corresponding to the folder being selected
        :param title: File browser window title
        :param row: Row location of file label within the Tkinter frame
        :param column: Column location of file label within the Tkinter frame
        :return:
        """

        try:
            if self.flabels['%s_display' % filekey]:
                label_pass = True
            else:
                label_pass = False
        except KeyError:
            self.flabels['%s_display' % filekey] = StringVar()
            label_pass = False

        d_out = str(tkinter.filedialog.askdirectory(initialdir='/', title=title))

        if d_out:
            self.files[filekey] = d_out
            file_loc_list = self.files[filekey].split('/')

            if len(file_loc_list[-1]) > 15:
                if len(file_loc_list) > 2:
                    self.flabels['%s_display' % filekey].set(file_loc_list[0] + '/.../' + file_loc_list[-1][0:15])
                else:
                    self.flabels['%s_display' % filekey].set(file_loc_list[0] + '/' + file_loc_list[-1][0:15])
            else:
                if len(file_loc_list) > 2:
                    self.flabels['%s_display' % filekey].set(file_loc_list[0] + '/.../' + file_loc_list[-1])
                else:
                    self.flabels['%s_display' % filekey].set(self.files[filekey])

            if label_pass:
                pass
            else:
                Label(
                    frame,
                    textvariable=self.flabels['%s_display' % filekey]
                ).grid(row=row, column=column, sticky=E, padx=5, pady=5)

        return

    def writeStartInfo(self):
        """
        Function writes user-selected start folders to start_info.txt file
        :return:
        """
        if 'home' in list(self.files.keys()) and 'bash' in list(self.files.keys()):
            info_handle = open('%s/start_info.txt' % os.path.dirname(__file__), 'w')
            if self.bash_option.get() == 0:
                bash_option = 'cygwin'
            else:
                bash_option = 'winbash'

            info_handle.write('home\t%s\nbash\t%s\nshell\t%s' % (self.files['home'], self.files['bash'], bash_option))

            info_handle.close()
            self.master.quit()
        else:
            message_text = 'Both "home" and "bash" directories must be selected.'
            self.submit_warning = Label(
                self.master,
                text=message_text, fg='#FF6D00')
            self.submit_warning.grid(row=15, columnspan=3, padx=5, pady=5)


def run():
    """
    Function runs start_info
    :return:
    """
    root = Tk()
    start_app(root)

    root.mainloop()
    root.destroy()



import gffutils
from Bio import SeqIO
from collections import defaultdict
import warnings
import subprocess


# Dictionary handling

def fastaToDict(infasta):
    """
    Takes FASTA file and returns a Python dictionary of IDs as keys and sequences as values
    :param infasta: File path for FASTA file
    :return: fastaDict: a dictionary of FASTA records
    """

    fasta_dict = defaultdict(list)

    for seq_record in SeqIO.parse(infasta, 'fasta'):
        fasta_dict[seq_record.id] = str(seq_record.seq)

    return fasta_dict


def dictToFasta(fasta_dict, filename, line_length=80):
    """
    Takes a Python dictionary of IDs as keys and sequences as values and outputs a FASTA file
    :param fasta_dict: Dictionary of FASTA IDs as keys, FASTA sequences as values
    :param filename: Name of FASTA file that will be written with this dictionary
    :param line_length:
    :return:
    """

    if type(fasta_dict) is dict and type(filename) is str:
        out_fasta = open(filename, "w")
        for key in list(fasta_dict.keys()):
            if line_length:
                line_start = 0
                line_end = line_length
                total_lines = (len(fasta_dict[key]) // line_length) + 1
                out_fasta.write('>%s\n' % key)
                for ii in range(total_lines):
                    out_fasta.write('%s\n' % fasta_dict[key][line_start:line_end])
                    line_start = line_end
                    line_end += line_length
            else:
                out_fasta.write('>%s\n%s\n' % (key, fasta_dict[key]))
    else:
        raise TypeError('dictToFasta accepts a dict input (arg 1) and str input file path (arg 2)')


# Match ID to GFF/GTF file

def isAttribute(gene_id, feature, parent=None, child=None, hierarchy=None):
    """
    Takes a gene/protein ID and checks a CDS feature and gene (parent) feature, if supplied, for matching IDs
    :param gene_id: gene/protein ID
    :param feature: feature that the ID will be compared to
    :param parent: parent feature, if applicable
    :param child:
    :param hierarchy: a list that gives the order by which an annotation record's different attributes will be tried for
    use as the primary name for a given ID
    :return: True/False, name/None: First returned value is True or False depending on whether the gene/protein ID
    provided matches the feature provided. Second value is the name that has been assigned to the feature, if it is a
    match. If it is not a match, the second argument is None.
    """

    # Initialize dictionaries for attributes
    attribute_dict = {'feature': {'gene': None, 'protein_id': None, 'Dbxref': None,  # GFF attributes
                                  'gene_id': None, 'gene_name': None, 'transcript_id': None, 'tss_id': None},  # GTF attributes
                      'parent': {'gene': None, 'Name': None, 'gene_synonym': None, 'locus_tag': None, 'Dbxref': None,
                                 'gene_id': None, 'gene_name': None, 'transcript_id': None, 'tss_id': None}}

    if not hierarchy:  # Set default hierarchy
        hierarchy = ['feature.gene', 'parent.gene', 'parent.Name', 'parent.gene_synonym', 'feature.protein_id',
                     'parent.locus_tag', 'feature.Dbxref', 'parent.Dbxref',
                     'feature.gene_id', 'feature.gene_name', 'parent.gene_id', 'parent.gene_name',
                     'feature.transcript_id', 'feature.tss_id', 'parent.transcript_id', 'parent.tss_id']

    if parent and not child:
        pass
    elif child and not parent:  # Reduce to single case based on symmetry of feature type possibilities
        parent = feature
        feature = child
    elif child and parent:
        warnings.warn('isAttribute does not take both parent and child for single feature, using parent as default')
        pass
    else:  # Handle case where feature type cannot be inferred from parent/child features
        attribute_dict['feature'] = {'gene': None, 'protein_id': None, 'Dbxref': None, 'Name': None,
                                     'gene_synonym': None, 'locus_tag': None,
                                     'gene_id': None, 'gene_name': None, 'transcript_id': None, 'tss_id': None}

    # Iterate through feature and parent gene records to search for all identifiers
    for key in list(attribute_dict['feature'].keys()):
        try:
            attribute_dict['feature'][key] = feature[key]
        except KeyError:
            try:
                hierarchy.remove('feature.%s' % key)
            except ValueError:
                pass

    if parent:
        for key in list(attribute_dict['parent'].keys()):
            try:
                attribute_dict['parent'][key] = parent[key]
            except KeyError:
                hierarchy.remove('parent.%s' % key)

    # Qualifiers that have been found are iterated through to look for matches
    for entry in hierarchy:
        try:
            [key_1, key_2] = entry.split('.')
            if key_2 == 'Dbxref' and attribute_dict[key_1][key_2]:
                identifier = [db_id.split(':')[1] for db_id in attribute_dict[key_1][key_2]]
            else:
                identifier = attribute_dict[key_1][key_2]

            if gene_id in identifier:
                name = attribute_dict[hierarchy[0].split('.')[0]][hierarchy[0].split('.')[1]][0]
                return True, name
        except TypeError:
            pass

    return False, None


def findID(gff_db, in_list, out_bed, feature_types=('CDS',), id_type='id', full_search=True):
    """
    Takes a path to a GFF3 file database (already created through gffutils.createdb function) and a file path for an ID
    list formatted as a single column list) of gene/protein identifiers, and outputs a .bed file of those records
    :param gff_db: A gffutils database that will be searched for features/attributes that match the IDs provided
    :param in_list: A single-column list of gene/protein IDs
    :param out_bed: The path and name of the output BED file
    :param feature_types: The types of features that should be examined. If left as None, the function searches all
    features
    :param id_type:
    :param full_search:
    :return:
    """

    db = gffutils.FeatureDB(gff_db)

    with open(in_list) as f_handle:
        id_list = [line.rstrip() for line in f_handle]

    outfile = open(out_bed, "w")
    not_found = []

    for gene_id in id_list:
        try:
            feature = db[gene_id]
            name = feature.id
            print('%s matches record for %s' % (gene_id, name))
            chromosome = feature.seqid
            start = feature.start
            end = feature.end
            score = feature.score
            strand = feature.strand

            outfile.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (chromosome, start, end, name, score, strand))
        except gffutils.exceptions.FeatureNotFoundError:
            not_found.append(gene_id)

    if full_search and not_found:
        if feature_types == ('',):
            db_list = list(db.all_features())
        else:
            db_list = list(db.all_features(featuretype=feature_types))

        for gene_id in not_found:
            for feature in db_list:
                condition = False
                name = None
                if id_type == 'id':
                    parent = None
                    try:
                        parent = list(db.parents(feature['ID'][0]))[0]
                    except KeyError:
                        try:
                            parent = list(db.parents(feature['gene_id'][0]))[0]
                        except IndexError:
                            pass
                    except IndexError:
                        pass

                    child = None
                    try:
                        child = list(db.children(feature['ID'][0]))[0]
                    except KeyError:
                        try:
                            child = list(db.children(feature['gene_id'][0]))[0]
                        except IndexError:
                            pass
                    except IndexError:
                        pass

                    condition, name = isAttribute(gene_id, feature, parent=parent, child=child)

                elif id_type == 'position':
                    try:
                        if feature.start == int(gene_id) or feature.end == int(gene_id):
                            condition = True
                            try:
                                name = feature['gene'][0]
                            except KeyError:
                                try:
                                    name = feature['gene_id'][0]
                                except KeyError:
                                    name = feature.id
                    except ValueError:
                        raise ValueError('If id_type is "position" list entries must be integers')
                else:
                    raise ValueError('id_type must be either "id" or "position"')

                if condition:
                    print('%s matches record for %s' % (gene_id, name))
                    chromosome = feature.seqid
                    start = feature.start
                    end = feature.end
                    score = feature.score
                    strand = feature.strand

                    outfile.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (chromosome, start, end, name, score, strand))
                    db_list.remove(feature)
                    break
                else:
                    db_list = db_list[1:] + [db_list[0]]

            else:
                print('No matching record found for gene ID: %s' % gene_id)

    elif not full_search and not_found:
        print('No matching record found for gene ID(s): %s' % ', '.join(not_found))
    else:
        print('All gene IDs successfully matched to features')

    outfile.close()


# BEDTools file helpers

def editBed(in_bed, out_bed, window=(-5, -5), sequence_length=12):
    """
    Takes input BED file (typically of targeted features/sequences produced by findID) and produces new BED file of
    enumerated sequences targeting the full window specified relative to the start codon
    :param in_bed: input BED file name
    :param out_bed: output BED file name
    :param window: the window of bases relative to the start codon for which antisense sequences will be designed
    :param sequence_length: the desired length of antisense sequences
    :return:
    """

    # Import bed file
    with open(in_bed) as fhandle:
        bed_list = [line.rstrip().split('\t') for line in fhandle]

    outfile = open(out_bed, "w")

    for record in bed_list:
        rec_num = 0
        ref_id = record[0]
        name = record[3]
        score = record[4]
        strand = record[5]

        if strand == '+':
            feat_start = int(record[1])
            slide_set = list(range(window[0] - 1, window[1]))

            for jj in slide_set:
                start = feat_start + jj
                end = start + sequence_length
                feat_id = name + '_%s' % str(rec_num)

                outfile.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (ref_id, str(start), str(end), feat_id, score, strand))
                rec_num += 1

        elif strand == '-':
            feat_start = int(record[2])
            slide_set = list(range(window[0], window[1] + 1))

            for jj in slide_set:
                start = feat_start + -1 * jj
                end = start - sequence_length
                feat_id = name + '_%s' % str(rec_num)

                outfile.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (ref_id, str(end), str(start), feat_id, score, strand))
                rec_num += 1

    outfile.close()


def processBedWindow(infile, outfile,
                     dist_filter=20,
                     ot_count_file=None,
                     check_homology=False,
                     homology_outfile=None,
                     feature_types=None,
                     log_mismatch=False):
    """
    Processes the output from a BEDTools window function to determine which feature-overlapping alignments are likely
    to cause antisense gene expression/mRNA translation inhibition.
    :param infile: Input BED file that will be analyzed
    :param outfile: Output .out file that will display filtered and processed results
    :param dist_filter: The distance downstream from the given feature start where expression/translation inhibition is
    still expected when an antisense molecule binds
    :param ot_count_file: Off-target count file that tabulates the number of inhibitory off-targets for each antisense
    sequence
    :param check_homology:
    :param homology_outfile:
    :param feature_types: The type of feature for which the user wants to find off-targets
    :return:
    """

    out = open(outfile, "w")
    out.write('PNA ID\tChromosome ID\tAlignment Strand\tAlignment Start\tFeature Start\tAlignment to STC\tFeature '
              'Info\n')
    ot_count = {}

    feature_check = True
    if feature_types is None:
        feature_types = ['CDS']
    elif feature_types == ['']:
        feature_check = False

    count_duplicates = {}
    potential_homology = []

    ot_handle = None
    h_handle = None
    line_number = 2

    if dist_filter == '':
        distance_pass = True
    else:
        distance_pass = False

    log_mismatch = False  #TODO
    with open(infile) as fhandle:
        for line in fhandle:
            record = line.split('\t')
            feature_type = record[8]

            # check for only input feature types, if feature types input not left blank
            if feature_type in feature_types:
                pass
            elif feature_check:
                continue
            else:
                pass

            pna_name = record[3]
            ref_id = record[0]
            strand = record[5]
            feat_info = record[14].split('\n')[0]

            if strand == '+':
                align_start = record[1]
                feat_start = record[9]
                distance = int(align_start) - int(feat_start) + 1
            elif strand == '-':
                align_start = record[2]
                feat_start = record[10]
                distance = int(feat_start) - int(align_start)
            else:
                raise IndexError('Error reading genome strand id')

            if distance_pass or distance < dist_filter:
                out_line = [pna_name, ref_id, strand, align_start, feat_start, str(distance), feat_info]

                # keep count of number of off-targets for each PNA
                if pna_name not in list(ot_count.keys()):
                    ot_count[pna_name] = 1
                    count_duplicates[pna_name] = {}
                    count_duplicates[pna_name][ref_id] = [feat_start]
                elif ref_id not in list(count_duplicates[pna_name].keys()):
                    count_duplicates[pna_name][ref_id] = []

                if feat_start not in count_duplicates[pna_name][ref_id]:
                    ot_count[pna_name] += 1
                    count_duplicates[pna_name][ref_id] += [feat_start]

                if check_homology:  # TODO: Improve this
                    if pna_name.split('_')[0].upper() in feat_info.upper():
                        potential_homology += [[pna_name, line_number, feat_info]]

                # write outfile
                for item in out_line:
                    out.write('%s\t' % str(item))
                out.write('\n')

                line_number += 1

    if ot_count_file:
        ot_handle = open(ot_count_file, 'w')
        for key in list(ot_count.keys()):
            ot_handle.write('%s\t%s\n' % (key, ot_count[key]))
    else:
        for key in list(ot_count.keys()):
            print(('%s\t%s' % (key, ot_count[key])))

    if check_homology and homology_outfile:
        h_handle = open(homology_outfile, 'w')
        h_handle.write('PNA ID\tOutfile Line Number\tFeature Info\n')
        for item in potential_homology:
            h_handle.write('%s\t%s\t%s\n' % (item[0], item[1], item[2]))
    elif check_homology and not homology_outfile:
        print('Potential alignments to homologous features:')
        for item in potential_homology:
            print('%s\t%s\n' % (item[0], item[2]))

    out.close()

    try:
        ot_handle.close()
    except AttributeError:
        pass

    try:
        h_handle.close()
    except AttributeError:
        pass

    return


# Miscellaneous functions

def revComp(sequence):
    """

    :param sequence:
    :return:
    """

    try:
        sequence = sequence.upper()
    except AttributeError:
        raise TypeError('rev_comp takes a nucleobase sequence string')

    rc_sequence = ''

    if 'T' in sequence and 'U' in sequence:
        raise ValueError('Mixed RNA and DNA sequences')
    elif 'U' in sequence:
        seq_type = 'RNA'
    else:
        seq_type = 'DNA'

    for letter in sequence[::-1]:
        if letter == 'A':
            if seq_type == 'DNA':
                rc_sequence += 'T'
            elif seq_type == 'RNA':
                rc_sequence += 'U'
        elif letter == 'T' or letter == 'U':
            rc_sequence += 'A'
        elif letter == 'C':
            rc_sequence += 'G'
        elif letter == 'G':
            rc_sequence += 'C'
        else:
            raise ValueError('Non-RNA or non-DNA base included')

    return rc_sequence


def rnafold(fasta, outfile=None, coordinates=None):
    """
    Returns either an tab-delimited output file or a dictionary of sequence names keyed to lists of the following
    values: [0] mRNA sequence, [1] folding plot, [2] free energy of folding, [3] (optional) secondary structure fraction
    of sub-sequence (as designated by coordinate list input)
    :param fasta: mRNA sequence file
    :param outfile: (optional) path to output file
    :param coordinates: (optional) coordinates of sub-sequence for which fractional folding will be returned
    :return:
    """

    fold_output = subprocess.check_output(['RNAfold', fasta]).decode('utf-8')
    out_dict = {}
    out_list = fold_output.split('\r\n')

    for ii in range(len(out_list)):
        if out_list[ii][0] == '>':
            name = out_list[ii][1:]
            sequence = out_list[ii + 1]
            fold_plot = out_list[ii + 2].split(' ')[0]
            energy = float(out_list[ii + 2].split(' ')[1][1:-1])

            out_dict[name] = [sequence, fold_plot, energy]
            if coordinates:
                try:
                    target_fold = fold_plot[coordinates[0]:coordinates[1]]
                    fraction = (target_fold.count('(') + target_fold.count(')')) / len(target_fold)
                    out_dict[name] += [fraction]
                except TypeError or IndexError:
                    Warning('Input variable "coordinates" must be a list of two integers')

    if outfile:
        with open(outfile, 'w') as out_handle:
            out_handle.write('Name\tSequence\tFold Plot\tEnergy')
            if coordinates:
                out_handle.write('\tTarget Fold Fraction')
            out_handle.write('\n')

            for name in out_dict.keys():
                out_handle.write('%s' % name)
                for element in out_dict[name]:
                    out_handle.write('\t%s' % element)
                out_handle.write('\n')
    else:
        return out_dict


def truncateFasta(fasta, gff_db, out_fasta, out_gff, out_info=None,
                  feature_types=('CDS',),
                  window=(-50, 50)):
    """
    Shortens FASTA file to relevant window of bases for PNA/ASO off-target searching
    :param fasta:
    :param gff_db:
    :param out_fasta:
    :param out_gff:
    :param out_info:
    :param feature_types:
    :param window:
    :return:
    """

    fasta_dict = fastaToDict(fasta)
    truncate_dict = {}

    db = gffutils.FeatureDB(gff_db)

    out_gff = open(out_gff, "w")
    out_info = open(out_info, "w")

    if out_info:
        info = (fasta, ', '.join(feature_types), str(window))
        info_text = 'Truncation settings: %s\nFeature Types: %s\nStart Range: %s' % info
        out_info.write(info_text)
        out_info.close()

    if feature_types == ['']:
        db_list = list(db.all_features())
    else:
        db_list = list(db.all_features(featuretype=feature_types))

    prev_feat_info = None
    for feature in db_list:
        feat_info = (feature.seqid, feature.start, feature.end, feature.strand)
        if feat_info == prev_feat_info:
            print('%s is a duplicate feature, skipping...' % feature.id)
            continue

        if feature.strand == '+':
            start = feature.start + window[0]
            end = feature.start + window[1]
            feature_start = -1 * window[0]
            feature_end = window[1] - window[0]
            if feature_start < 0:
                feature_start = 1
        elif feature.strand == '-':
            start = feature.end + window[0]
            end = feature.end + window[1]
            feature_start = 1
            feature_end = -1 * window[0]
            if feature_start < 0:
                feature_start = 0
        else:
            warnings.warn('Skipping feature %s, no strand designation' % feature.id)
            continue

        truncate_dict[feature.seqid + '_%s' % feature.id] = fasta_dict[feature.seqid][start:end]

        feature_list = str(feature).split('\t')
        feature_list[0] = feature.seqid + '_%s' % feature.id

        feature_list[3] = str(feature_start)
        feature_list[4] = str(feature_end)

        out_gff.write('\t'.join(feature_list) + '\n')

        prev_feat_info = feat_info

    out_gff.close()
    dictToFasta(truncate_dict, out_fasta)
    return


def checkHomology(outfile, homology_outfile=None):
    """
    Function to check homology outside of the processBedWindow function
    :return:
    """

    potential_homology = []
    line_number = 2
    h_handle = None

    with open(outfile) as fhandle:
        for line in fhandle:
            record = line.split('\t')
            pna_name = record[0]
            gene_target = pna_name.split('_')[0]
            feat_info = record[6]

            if gene_target.upper() in feat_info.upper():
                potential_homology += [[pna_name, line_number, feat_info]]

            line_number += 1

    if homology_outfile:
        h_handle = open(homology_outfile, 'w')
        for item in potential_homology:
            h_handle.write('%s\t%s\t%s\n' % (item[0], item[1], item[2]))
    else:
        print('Potential alignments to homologous features:')
        for item in potential_homology:
            print('%s\t%s\t%s\n' % (item[0], item[1], item[2]))

    try:
        h_handle.close()
    except AttributeError:
        pass

    return



# Applies the arguments submitted to the main PNA Finder dialog to the internal PNA Finder functions and external
# alignment programs

from . import string_genes as sg
from .align_tools import *
from . import pna_tools
from . import pna_solubility as sol
import gffutils
import os
import subprocess
import shutil


def getSequences(submit_dict):
    """
    Executes the PNA Finder Get Sequences function using arguments provided by the dictionary submit_dict:
    :param submit_dict:
        get_sequences submit_dict keys
            startup keys
                bash        -   str
                shell_type  -   str
            job parameter keys
                start       -   int
                end         -   int
                pna_length  -   int
                feature_type -  str
            file keys
                gff_option  -   int
                id_list     -   str
                assembly    -   str
                gff         -   str
                output      -   str
            option keys
                STRING      -   int (optional)
                string_id   -   int (optional)
                warnings    -   int (optional)
    (see manual.pdf in package root directory for more info)
    :return:
    """

    bash_dir = submit_dict['bash']
    shell_type = submit_dict['shell_type']

    # Retrieve job parameters
    window = (submit_dict['start'], submit_dict['end'])
    length = submit_dict['length']

    feature_types = submit_dict['feature_type'].split(',')
    for ii in range(len(feature_types)):
        feature_types[ii] = feature_types[ii].strip()
    feature_types = tuple(feature_types)

    gff_option = submit_dict['gff_option']
    warnings_option = submit_dict['warnings']
    temperature_option = submit_dict['temp_option']
    string_option = submit_dict['STRING']

    species = None
    if submit_dict['STRING'] == 1:
        try:
            species = submit_dict['string_id']
        except KeyError:
            raise KeyError('Species Taxonomy ID must be specified')

    # Retrieve job file paths
    id_list = submit_dict['id_list']
    assembly = submit_dict['assembly']
    annotation = submit_dict['gff']

    # Initialize folder and files
    out_dir = submit_dict['output']
    out_base = out_dir
    out_num = 0
    folder_check = True
    name_change = False

    while folder_check:
        try:
            os.mkdir(out_dir)
            folder_check = False
            os.mkdir('%s/error_files' % out_dir)
        except WindowsError:
            name_change = True
            out_dir = out_base + ' (%s)' % str(out_num + 1)
            out_num += 1

    if name_change:
        print('Directory %s already exists, job results written to directory %s' % (out_base, out_dir))

    filebase = submit_dict['id_list'].split('/')[-1].split('.')[0]

    #   Parse and execute gff option
    if gff_option == 0:
        dbfile = annotation.rsplit('.', 1)[0] + '.db'
        gffutils.create_db(annotation, dbfn=dbfile, id_spec=["gene", "Name"],
                           force=True, keep_order=True,
                           merge_strategy='merge', sort_attribute_values=True)
    elif gff_option == 1:
        dbfile = annotation
    else:
        raise ValueError('Something has gone horribly wrong!')

    bedfile = out_dir + '/' + filebase + '.bed'
    bedfile_edited = out_dir + '/' + filebase + '.edit.bed'
    fasta_outfile = out_dir + '/' + filebase + '.fa'
    sequence_outfile = out_dir + '/' + filebase + '.out'
    string_outfile = None
    if string_option:
        string_outfile = out_dir + '/' + 'string.out'

    # Check version for BEDTools
    bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bedtools --version']
    version_call = subprocess.check_output(bash_check).decode('utf-8')
    bedtools_version = version_call.split(' ')[1].split('\n')[0]

    # Run pna_tools programs
    pna_tools.findID(dbfile, id_list, bedfile, feature_types=feature_types, id_type='id')
    pna_tools.editBed(bedfile, bedfile_edited, window=window, sequence_length=length)

    # Run BEDTools getfasta
    flags = ['-name', '-s']
    bedtools('getfasta',
             [assembly, bedfile_edited],
             outfile=fasta_outfile,
             flags=flags,
             bash_sub=shell_type, bash_dir=bash_dir,
             err_file='%s/error_files/bedtools.stderr' % out_dir,
             version=bedtools_version)

    # Run sequence solubility checks and STRING interaction finder
    sequence_dict = pna_tools.fastaToDict(fasta_outfile)
    sequence_handle = open(sequence_outfile, 'w')
    sequence_handle.write('Name\tSequence')

    if warnings_option == 1:
        sequence_handle.write('\tWarnings')
    if temperature_option == 1:
        sequence_handle.write('\tTm (DNA/ssPNA)')
    sequence_handle.write('\n')

    string_handle = None
    if submit_dict['STRING'] == 1:
        string_handle = open(string_outfile, 'w')
        string_handle.write('Gene\tNetwork Nodes\tNetwork Edges\tNetwork Genes\n')
    string_names = []

    if not warnings_option and not temperature_option and not string_option:
        pass
    else:
        for key in list(sequence_dict.keys()):
            name = key.rsplit('_', 1)[0]
            sequence = pna_tools.revComp(sequence_dict[key])
            sequence_handle.write('%s\t%s\t' % (key, sequence))

            if warnings_option:
                warnings = sol.checkWarnings(sequence)
                if warnings:
                    sequence_handle.write(', '.join(warnings))

            if temperature_option:
                dH, dS, dG, temp_dna, temp_pna = sol.calculateTmPNA(sequence)
                sequence_handle.write('\t%s' % round(temp_pna, 1))

            sequence_handle.write('\n')

            if string_option and name not in string_names:
                net_genes, nodes, edges = sg.STRING_net(name, species, exclude_tm=True)
                string_handle.write('%s\t%s\t%s' % (name, nodes, edges))
                try:
                    string_handle.write('\t%s' % ','.join(net_genes))
                except TypeError:
                    pass
                string_handle.write('\n')
                string_names.append(name)

    sequence_handle.close()
    try:
        string_handle.close()
    except AttributeError:
        pass

    return


def findOffTargets(submit_dict):
    """
    Executes the PNA Finder Find Off-Targets function using arguments provided by the dictionary submit_dict:
    :param submit_dict:
        find_off_targets submit_dict keys
            startup keys
                bash        -   str
                shell_type  -   str
            job parameter keys
                start       -   int
                end         -   int
                mismatch    -   int
                length      -   int
                feature_type -  str
            file keys
                index_option -  int
                targets     -   str
                assembly    -   str
                gff         -   str
                output      -   str
            option keys
                log_mismatch -  int (optional)
                count       -   int (optional)
                homology    -   int (optional)
                remove_files -  int (optional)
    (see manual.pdf in package root directory for more info)
    :return:
    """

    bash_dir = submit_dict['bash']
    shell_type = submit_dict['shell_type']

    # Retrieve job parameters
    window = (submit_dict['start'], submit_dict['end'])
    mismatches = submit_dict['mismatch']
    index_option = submit_dict['index_option']
    count = submit_dict['count']
    check_homology = submit_dict['homology']
    remove_files = submit_dict['remove_files']
    small_genome = False  # TODO

    feature_types = submit_dict['feature_type'].split(',')
    for ii in range(len(feature_types)):
        feature_types[ii] = feature_types[ii].strip()

    # Retrieve job file paths
    targets = submit_dict['targets']
    assembly = submit_dict['assembly']
    annotation = submit_dict['gff']

    # Initialize folder and files
    out_dir = submit_dict['output']
    out_base = out_dir
    out_num = 0
    folder_check = True
    name_change = False

    while folder_check:
        try:
            os.mkdir(out_dir)
            folder_check = False
            os.mkdir('%s/error_files' % out_dir)
        except WindowsError:
            name_change = True
            out_dir = out_base + ' (%s)' % str(out_num + 1)
            out_num += 1

    if name_change:
        print('Directory %s already exists, job results written to directory %s' % (out_base, out_dir))

    filebase = submit_dict['targets'].split('/')[-1].split('.')[0]
    os.mkdir(out_dir + '/sambam/')

    # Parse and execute index option
    if index_option == 0:
        # Check version
        bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bowtie-build --version']
        version_call = subprocess.check_output(bash_check).decode('utf-8')
        bt_build_version = version_call.split('\n')[0].split()[-1]

        index_base = assembly.rsplit('.', 1)[0]
        bowtie_build(assembly, index_base,
                     bash_sub=shell_type, bash_dir=bash_dir,
                     version=bt_build_version,
                     err_file='%s/error_files/bowtie-build.stderr' % out_dir)
    elif index_option == 1:
        index_base = assembly.rsplit('.', 2)[0]
        if index_base.split('.')[-1] == 'rev':
            index_base = index_base.rsplit('.', 1)[0]
    else:
        raise ValueError('Something has gone horribly wrong!')

    # Check version and run Bowtie
    bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bowtie --version']
    version_call = subprocess.check_output(bash_check).decode('utf-8')
    bt_version = version_call.split('\n')[0].split()[-1]

    # Find PNA length
    targets_dict = pna_tools.fastaToDict(targets)
    length_dict = {}
    sam_dict = {}

    for key in targets_dict:
        length = str(len(targets_dict[key]))
        try:
            length_dict[length][key] = targets_dict[key]
            if '%s.rev' % key not in length_dict:  # Add reverse sequence for N to 5' parallel alignment
                length_dict[length]['%s.rev' % key] = targets_dict[key][::-1]
        except KeyError:
            length_dict[length] = {}
            length_dict[length][key] = targets_dict[key]
            if '%s.rev' % key not in length_dict:  # Add reverse sequence for N to 5' parallel alignment
                length_dict[length]['%s.rev' % key] = targets_dict[key][::-1]

    # Run Bowtie for each length of PNA, each mismatch tolerance
    for key in length_dict:
        if len(length_dict) > 1:
            print('Separating FASTA file for PNA length %s...' % key)

        temp_targets = out_dir + '/' + filebase + '.%smer.fa' % key
        pna_tools.dictToFasta(length_dict[key], temp_targets)
        temp_len = key

        flags = ['-f', '-l', str(key), '-a']
        if small_genome:
            flags += ['-y']

        for mismatch in mismatches:
            flags += ['-n', str(mismatch)]
            sambase = out_dir + '/sambam/%sMM/' % mismatch + filebase
            os.mkdir('%s/sambam/%sMM/' % (out_dir, mismatch))
            os.mkdir('%s/error_files/%sMM/' % (out_dir, mismatch))

            sam_dict[mismatch] = []
            temp_sam = sambase + '.%smer.sam' % key
            temp_bam = sambase + '.%smer.bam' % key
            temp_sbam = sambase + '.%smer.sorted.bam' % key
            temp_bed = sambase + '.%smer.bed' % key
            out_bed = sambase + '.bed'
            sam_dict[mismatch] += [[temp_len, temp_sam, temp_bam, temp_sbam, temp_bed, out_bed]]

            bowtie(temp_targets,
                   index_base,
                   temp_sam,
                   flags,
                   bash_sub=shell_type, bash_dir=bash_dir,
                   err_file='%s/error_files/%sMM/bowtie.%smer.stderr' % (out_dir, mismatch, key),
                   version=bt_version)

    # Check SAMTools version
    bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'samtools --version']
    version_call = subprocess.check_output(bash_check).decode('utf-8')
    samtools_version = version_call.split(' ')[1].split('\n')[0]

    bed_handle = None
    bed_dict = {}

    for mismatch in sam_dict:
        first = True
        for sam in sam_dict[mismatch]:
            length = sam[0]
            samfile = sam[1]
            bamfile = sam[2]
            sbamfile = sam[3]
            temp_bed = sam[4]
            out_bed = sam[5]

            samtools('view', samfile, outfile=bamfile, flags=['-b'], bash_sub=shell_type, bash_dir=bash_dir,
                     err_file='%s/error_files/%sMM/samtools_view.%smer.stderr' % (out_dir, mismatch, length),
                     version=samtools_version)
            samtools('sort', bamfile, outfile=sbamfile, bash_sub=shell_type, bash_dir=bash_dir,
                     err_file='%s/error_files/%sMM/samtools_sort.%smer.stderr' % (out_dir, mismatch, length),
                     version=samtools_version)
            samtools('index', sbamfile, bash_sub=shell_type, bash_dir=bash_dir,
                     err_file='%s/error_files/%sMM/samtools_index.%smer.stderr' % (out_dir, mismatch, length),
                     version=samtools_version)

            # Check version and run BEDTools
            bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bedtools --version']
            version_call = subprocess.check_output(bash_check).decode('utf-8')
            bedtools_version = version_call.split(' ')[1].split('\n')[0]

            bedtools_function = 'window'
            infiles = [sbamfile, annotation]
            flags = ['-l', str(-1 * window[0]), '-r', str(-1 * window[0]), '-sw', '-sm', '-bed']

            bedtools(bedtools_function,
                     infiles,
                     outfile=temp_bed,
                     flags=flags,
                     bash_sub=shell_type, bash_dir=bash_dir,
                     err_file='%s/error_files/%sMM/bedtools.%smer.stderr' % (out_dir, mismatch, length),
                     version=bedtools_version)

            if len(sam_dict[mismatch]) > 1:
                if first:
                    bed_handle = open(out_bed, 'w')
                    bed_dict[mismatch] = out_bed
                    first = False
                with open(temp_bed) as temp_bed_handle:
                    for line in temp_bed_handle:
                        bed_handle.write(line)
                    temp_bed_handle.close()
                os.remove(temp_bed)
            else:
                os.rename(temp_bed, out_bed)
                bed_dict[mismatch] = out_bed

        try:
            bed_handle.close()
        except AttributeError:
            pass

    # Filter alignments
    first = True
    for mismatch in bed_dict:
        bedfile = bed_dict[mismatch]
        outfile = out_dir + '/' + filebase + '.%smm.out' % mismatch
        if count:
            countfile = out_dir + '/' + filebase + '.%smm.count' % mismatch
        else:
            countfile = None

        if check_homology and first:
            homology_outfile = out_dir + '/' + 'homology.out'
            first = False
        else:
            check_homology = False
            homology_outfile = None

        pna_tools.processBedWindow(bedfile,
                                   outfile,
                                   dist_filter=window[1],
                                   ot_count_file=countfile,
                                   homology_outfile=homology_outfile,
                                   check_homology=check_homology,
                                   feature_types=feature_types)

    if remove_files:
        shutil.rmtree('%s/error_files' % out_dir)
        shutil.rmtree('%s/sambam' % out_dir)

    return


def sequenceWarnings(submit_dict):
    """
    Executes the PNA Finder Check Sequence Warnings function using arguments provided by the dictionary submit_dict:
    :param submit_dict:
       check_warnings submit_dict keys
            file keys
                pna_option  -   int
                pna_list    -   str
            option keys
                ss_tm       -  int (optional)
    (see manual.pdf in package root directory for more info)
    :return:
    """

    if submit_dict['pna_option'] == 0:
        pna_targets = submit_dict['pna_list']
        sequence_dict = pna_tools.fastaToDict(pna_targets)
    elif submit_dict['pna_option'] == 1:
        pna_list = submit_dict['pna_list']
        sequence_dict = dict()
        with open(pna_list) as pna_handle:
            for line in pna_handle:
                [key, value] = line.rstrip().split('\t')
                sequence_dict[key] = pna_tools.revComp(value)
    else:
        raise ValueError('Something has gone horribly wrong!')

    temperature_option = submit_dict['temp_option']

    # Initialize folder and files
    out_dir = submit_dict['output']
    out_base = out_dir
    out_num = 0
    folder_check = True
    name_change = False

    while folder_check:
        try:
            os.mkdir(out_dir)
            folder_check = False
            os.mkdir('%s/error_files' % out_dir)
        except WindowsError:
            name_change = True
            out_dir = out_base + ' (%s)' % str(out_num + 1)
            out_num += 1

    if name_change:
        print('Directory %s already exists, job results written to directory %s' % (out_base, out_dir))

    filebase = submit_dict['pna_list'].split('/')[-1].split('.')[0]

    sequence_outfile = out_dir + '/' + filebase + '.out'
    sequence_handle = open(sequence_outfile, 'w')
    sequence_handle.write('Name\tSequence\tWarnings')

    if temperature_option:
        sequence_handle.write('\tTm (DNA/ssPNA)')

    sequence_handle.write('\n')

    for key in list(sequence_dict.keys()):
        sequence = pna_tools.revComp(sequence_dict[key])
        sequence_handle.write('%s\t%s\t' % (key, sequence))

        warnings = sol.checkWarnings(sequence)
        if warnings:
            sequence_handle.write(', '.join(warnings))

        if temperature_option:
            dH, dS, dG, temp_dna, temp_pna = sol.calculateTmPNA(sequence)
            sequence_handle.write('\t%s' % round(temp_pna, 1))

        sequence_handle.write('\n')

    sequence_handle.close()
    return



# Module that allows PNA Finder to access STRING database

import sys
import urllib


def STRING_int(genes, species):
    """
    Uses the STRING database to test whether a gene network has more interactions than expected. Parts of this function
    are provided by the STRING database at https://string-db.org/cgi/help.pl
    :param genes: list of genes in interaction network
    :param species: NCBI Taxonomy ID for species
    :return:
    """

    string_api_url = "https://string-db.org/api"
    output_format = "tsv-no-header"
    method = "ppi_enrichment"

    # Construct the request

    request_url = string_api_url + "/" + output_format + "/" + method + "?"
    request_url += "identifiers=" + "%0d".join(genes)
    request_url += "&" + "species=" + species

    # Call STRING

    try:
        response = urllib.request.urlopen(request_url)
    except urllib.error.HTTPError as err:
        error_message = err.read()
        print(error_message)
        sys.exit()

    # Read and parse the results

    result = response.readline()

    while result:
        print(result)
        result = response.readline()


def STRING_annotation(genes, species, outfile, pubmed=0):
    """

    :param gene: list of genes for functional annotation analysis
    :param species:
    :param pubmed:
    :return:
    """

    string_api_url = "https://string-db.org/api"
    output_format = "tsv-no-header"
    method = "functional_annotation"

    # Construct the request

    request_url = string_api_url + "/" + output_format + "/" + method + "?"
    request_url += "identifiers=%s" % '%0d'.join(genes)
    request_url += "&" + "species=%s" % species
    request_url += "&" + "allow_pubmed=%s" % pubmed

    try:
        response = urllib.request.urlopen(request_url)
    except urllib.error.HTTPError as err:
        error_message = err.read()
        print(error_message)
        return

    with open(outfile, 'w') as out_handle:
        for line in response.readlines():
            out_handle.write(line.decode('utf-8'))
    out_handle.close()

    return


def STRING_net(gene, species, max_nodes=500, required_score=400, exclude_tm=False):
    """
    Provides a gene interaction network for a given gene in a given species. Parts of this function are provided by the
    STRING database at https://string-db.org/cgi/help.pl
    :param gene: central gene of the interaction network
    :param species: NCBI Taxonomy ID for species
    :param max_nodes: The maximum number of nodes (proteins) to be included in the interaction network
    :param required_score: The minimum score (0-1000) for an interaction to be included in the network
    :param exclude_tm: Option to exclude text mining-based interaction results
    :return:
        net_genes: the genes within the interaction network
        nodes: the number of genes within the network
        edges: the number of interactions between the genes in the network
    """

    string_api_url = "https://string-db.org/api"
    output_format = "tsv-no-header"
    method = "network"

    # Construct the request

    request_url = string_api_url + "/" + output_format + "/" + method + "?"
    request_url += "identifiers=%s" % gene
    request_url += "&" + "species=%s" % str(species)
    request_url += "&" + "add_nodes=" + str(max_nodes) + "&" + "required_score=" + str(required_score)

    try:
        response = urllib.request.urlopen(request_url)
    except urllib.error.HTTPError as err:
        error_message = err.read()
        print(error_message)
        return None, None, None

    # Read and parse the results

    line = response.readline().decode('utf-8')
    interaction_list = []
    direct_interaction_list = []
    net_genes = []
    edges = 0

    while line:
        line_list = line.strip().split("\t")
        g1, g2 = line_list[2], line_list[3]
        if exclude_tm:
            prior = 0.041
            score_list = [float(line_list[ii]) for ii in range(6, len(line_list) - 1) if line_list[ii] != '0']
            score_list_nop = [(float(score) - prior)/(1 - prior) for score in score_list]
            nop_prod = 1
            for score_nop in score_list_nop:
                if score_nop != 0:
                    nop_prod = nop_prod * (1 - score_nop)
            s_tot_nop = 1 - nop_prod
            s_tot = s_tot_nop + prior * (1 - s_tot_nop)

            if s_tot < float(required_score)/1000:
                line = response.readline().decode('utf-8')
                continue

            interaction_list += [line_list]
            if gene in [g1, g2]:
                direct_interaction_list += [g1, g2]

        else:
            interaction_list += [line_list]
            if gene in [g1, g2]:
                direct_interaction_list += [g1, g2]

        line = response.readline().decode('utf-8')

    pairs = []
    for interaction in interaction_list:
        g1, g2 = interaction[2], interaction[3]

        if [g1, g2] in pairs or [g2, g1] in pairs:
            continue
        else:
            pairs += [[g1, g2]]

        if g1 in direct_interaction_list and g2 in direct_interaction_list:
            if g1 not in net_genes:
                net_genes.append(str(g1))
            if g2 not in net_genes:
                net_genes.append(str(g2))
            edges += 1
        else:  # For clustering coefficient calculation
            interaction_list.remove(interaction)

    nodes = len(net_genes)

    return net_genes, nodes, edges



import subprocess


def bashToWindows(path):
    """
    Provides workaround for Windows bash shell path handling, allowing Python subprocess.call functions to work on
    bash shells in Windows
    :param path: Cygwin or Windows bash path (e.g. str starting with '/cygdrive/c/...' or '/mnt/c/...'
    :return: Windows path (e.g. str starting with 'C:/...')
    """

    if type(path) is str:
        path_list = path.split('/')
        if path_list[1] == 'cygdrive':
            winpath = path_list[2].upper() + ':/' + '/'.join(path_list[3:])
            return winpath
        elif path_list[1] == 'mnt':
            winpath = path_list[2].upper() + ':/' + '/'.join(path_list[3:])
            return winpath
        else:
            raise ValueError('bashToWindows takes Windows bash shell file path, e.g. /cygdrive/c/... or /mnt/c/...')
    else:
        raise TypeError('bashToWindows takes string path input')


def windowsToBash(path, shell_type):
    """
    Provides workaround for Windows bash shell path handling, allowing Python subprocess.call functions to work on
    bash shells in Windows
    :param path: Windows file path
    :param shell_type: 'cygwin' or 'winbash'
    :return:
    """

    if type(path) is str:
        if '/' in path:
            path_list = path.split('/')
        elif '\\' in path:
            path_list = path.split('\\')
        else:
            raise ValueError('windowsToBash takes Windows file path, e.g. C:/...')

        if shell_type == 'cygwin':
            try:
                bashpath = '/cygdrive/' + path_list[0][0].lower() + '/' + '/'.join(path_list[1:])
                return bashpath
            except IndexError:
                raise ValueError('windowsToBash takes Windows file path, e.g. C:/...')
        elif shell_type == 'winbash':
            try:
                bashpath = '/mnt/' + path_list[0][0].lower() + '/' + '/'.join(path_list[1:])
                return bashpath
            except IndexError:
                raise ValueError('windowsToBash takes Windows file path, e.g. C:/...')
        else:
            raise ValueError('windowsToBash takes Windows file path, e.g. C:/...')
    else:
        raise TypeError('windowsToBash takes string path input')


def bowtie_build(fasta, refpath, err_file='bowtie-build.stderr',
                 version=None,
                 bash_sub=False, bash_dir='/',):
    """
    Construct bowtie indices (.ebwt files)
    :param fasta: Path to FASTA containing reference sequences
    :param refpath: Path to index base (e.g. C:/directory/e_coli_MG1655)
    :param err_file: File path to where output is written
    :param version: Enforces bowtie version number (e.g., '1.2.3')
    :param bash_sub: False/cygwin/winbash
    :param bash_dir: Location of bash.exe, if bash_sub not False
    :return:
    """

    try:
        if not bash_sub:
            version_call = subprocess.check_output(['bowtie-build', '--version']).decode('utf-8')
        else:
            bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bowtie-build --version']
            version_call = subprocess.check_output(bash_check).decode('utf-8')
    except OSError:
        raise RuntimeError('bowtie-build not found; check if it is installed and in $PATH\n')

    local_version = version_call.split('\n')[0].split()[-1]
    assert version == local_version, 'bowtie-build version incompatibility %s != %s' % (version, local_version)

    # Change file formats for winbash option
    if bash_sub == 'winbash':
        fasta = windowsToBash(fasta, 'winbash')
        refpath = windowsToBash(refpath, 'winbash')

    # Make sure spaces in files are escaped for bash
    if ' ' in fasta:
        fasta = '"' + fasta + '"'

    if ' ' in refpath:
        refpath = '"' + refpath + '"'

    # Initialize error/output file
    err_handle = open(err_file, "w")

    # Call bowtie-build
    if not bash_sub:
        subprocess.call(['bowtie-build', '-f', fasta, refpath],
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)
    else:
        bash_call = ['%s/bash' % bash_dir, '--login', '-c', 'bowtie-build -f %s %s' % (fasta, refpath)]
        subprocess.call(bash_call,
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)

    err_handle.close()


def bowtie(fasta, refpath, samfile, flags, err_file='bowtie.stderr',
           version=None,
           bash_sub=False, bash_dir='/'):
    """
    Call bowtie2-align
    :param fasta: fasta file for alignment
    :param refpath: Path to bowtie index files (.bt2)
    :param samfile: path to samfile output
    :param flags: A list of bowtie flags such as ['-f', '-N', '0', '-L', '12', '-a']
    :param err_file: File path to where output is written
    :param version: Enforces bowtie version number
    :param bash_sub: False/cygwin/winbash
    :param bash_dir: Location of bash.exe, if bash_sub not False
    :return:
    """

    # Check that we have access to bowtie
    try:
        if not bash_sub:
            version_call = subprocess.check_output(['bowtie', '--version']).decode('utf-8')
        else:
            bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bowtie --version']
            version_call = subprocess.check_output(bash_check).decode('utf-8')
    except OSError:
        raise RuntimeError('bowtie not found; check if it is installed and in $PATH\n')

    # Check that version is the expected version
    local_version = version_call.split('\n')[0].split()[-1]
    assert version == local_version, 'bowtie version incompatibility %s != %s' % (version, local_version)

    # Change file formats for winbash option
    if bash_sub == 'winbash':
        fasta = windowsToBash(fasta, 'winbash')
        refpath = windowsToBash(refpath, 'winbash')
        samfile = windowsToBash(samfile, 'winbash')

    # Make sure spaces in files are escaped for bash
    if ' ' in fasta:
        fasta = '"' + fasta + '"'

    if ' ' in refpath:
        refpath = '"' + refpath + '"'

    if ' ' in samfile:
        samfile = '"' + samfile + '"'

    # Initialize error/output file
    err_handle = open(err_file, "w")

    # Call bowtie2
    bowtie_args = ['bowtie', refpath, fasta, '-S', samfile] + list(flags)

    if not bash_sub:
        subprocess.call(bowtie_args,
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)
    else:
        bash_call = ['%s/bash' % bash_dir, '--login', '-c'] + [' '.join(bowtie_args)]
        subprocess.call(bash_call,
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)

    err_handle.close()


def bowtie2_build(fasta, refpath, err_file='bowtie2-build.stderr',
                  version=None,
                  bash_sub=False, bash_dir='/',):
    """
    Construct bowtie2 indices (.bt2 files)
    :param fasta: Path to FASTA containing reference sequences
    :param refpath: Path to index base (e.g. C:/directory/e_coli_MG1655)
    :param err_file: File path to where output is written
    :param version: Enforces bowtie2 version number (e.g., '2.2.1')
    :param bash_sub: False/cygwin/winbash
    :param bash_dir: Location of bash.exe, if bash_sub not False
    :return:
    """

    try:
        if not bash_sub:
            version_call = subprocess.check_output(['bowtie2-build', '--version']).decode('utf-8')
        else:
            bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bowtie2-build --version']
            version_call = subprocess.check_output(bash_check).decode('utf-8')
    except OSError:
        raise RuntimeError('bowtie2-build not found; check if it is installed and in $PATH\n')

    local_version = version_call.split('\n')[0].split()[-1]
    assert version == local_version, 'bowtie2-build version incompatibility %s != %s' % (version, local_version)

    # Change file formats for winbash option
    if bash_sub == 'winbash':
        fasta = windowsToBash(fasta, 'winbash')
        refpath = windowsToBash(refpath, 'winbash')

    # Make sure spaces in files are escaped for bash
    if ' ' in fasta:
        fasta = '"' + fasta + '"'

    if ' ' in refpath:
        refpath = '"' + refpath + '"'

    # Initialize error/output file
    err_handle = open(err_file, "w")

    # Call bowtie2-build
    if not bash_sub:
        subprocess.call(['bowtie2-build', '-f', fasta, refpath],
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)
    else:
        bash_call = ['%s/bash' % bash_dir, '--login', '-c', 'bowtie2-build -f %s %s' % (fasta, refpath)]
        subprocess.call(bash_call,
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)

    err_handle.close()


def bowtie2(fasta, refpath, samfile, flags, err_file='bowtie2.stderr',
            version=None,
            bash_sub=False, bash_dir='/'):
    """
    Call bowtie2-align
    :param fasta: fasta file for alignment
    :param refpath: Path to bowtie2 index files (.bt2)
    :param samfile: path to samfile output
    :param flags: A list of bowtie2 flags such as ['-f', '-N', '0', '-L', '12', '-a']
    :param err_file: File path to where output is written
    :param version: Enforces bowtie2 version number
    :param bash_sub: False/cygwin/winbash
    :param bash_dir: Location of bash.exe, if bash_sub not False
    :return:
    """

    # Check that we have access to bowtie2
    try:
        if not bash_sub:
            version_call = subprocess.check_output(['bowtie2', '--version']).decode('utf-8')
        else:
            bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bowtie2 --version']
            version_call = subprocess.check_output(bash_check).decode('utf-8')
    except OSError:
        raise RuntimeError('bowtie2 not found; check if it is installed and in $PATH\n')

    # Check that version is the expected version
    local_version = version_call.split('\n')[0].split()[-1]
    assert version == local_version, 'bowtie2 version incompatibility %s != %s' % (version, local_version)

    # Change file formats for winbash option
    if bash_sub == 'winbash':
        fasta = windowsToBash(fasta, 'winbash')
        refpath = windowsToBash(refpath, 'winbash')
        samfile = windowsToBash(samfile, 'winbash')

    # Make sure spaces in files are escaped for bash
    if ' ' in fasta:
        fasta = '"' + fasta + '"'

    if ' ' in refpath:
        refpath = '"' + refpath + '"'

    if ' ' in samfile:
        samfile = '"' + samfile + '"'

    # Initialize error/output file
    err_handle = open(err_file, "w")

    # Call bowtie2
    bowtie_args = ['bowtie2', '-x', refpath, '-U', fasta, '-S', samfile] + list(flags)

    if not bash_sub:
        subprocess.call(bowtie_args,
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)
    else:
        bash_call = ['%s/bash' % bash_dir, '--login', '-c'] + [' '.join(bowtie_args)]
        subprocess.call(bash_call,
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)

    err_handle.close()


def samtools(function, infile, outfile=None, flags=(), err_file='samtools.stderr',
             version=None,
             bash_sub=False, bash_dir='/'):
    """
    Call samtools functions view, sort, or index
    :param function: view, sort, index
    :param infile: file path to input file
    :param outfile: file path to output file if needed (e.g. outfile=None for index function)
    :param flags: list of additional flags that can be passed to the given function
    :param err_file: samtools output/error file
    :param version: Enforces samtools version number
    :param bash_sub: False/cygwin/winbash
    :param bash_dir: Location of bash.exe, if bash_sub not False
    :return:
    """

    # Check that we have access to samtools
    try:
        if not bash_sub:
            version_call = subprocess.check_output(['samtools', '--version']).decode('utf-8')
        else:
            bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'samtools --version']
            version_call = subprocess.check_output(bash_check).decode('utf-8')
    except OSError:
        raise RuntimeError('samtools not found; check if it is installed and in $PATH\n')

    # Check that version is the expected version
    local_version = version_call.split(' ')[1].split('\n')[0]
    assert version == local_version, 'samtools version incompatibility %s != %s' % (version, local_version)

    # Check that chosen function is supported
    if function not in ['view', 'sort', 'index']:
        raise Exception('Function %s not currently supported by python samtools wrapper' % function)

    # Change file formats for winbash option
    if bash_sub == 'winbash':
        infile = windowsToBash(infile, 'winbash')

        if outfile:
            outfile = windowsToBash(outfile, 'winbash')

    # Make sure spaces in files are escaped for bash
    if ' ' in infile:
        infile = '"' + infile + '"'

    if outfile and ' ' in outfile:
        outfile = '"' + outfile + '"'

    # Initialize arguments
    samtools_args = ['samtools', function, infile] + list(flags)

    if outfile:
        samtools_args += ['-o', outfile]

    # Initialize error file
    err_handle = open(err_file, "w")

    # Call samtools
    if not bash_sub:
        subprocess.call(samtools_args,
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)
    else:
        bash_call = ['%s/bash' % bash_dir, '--login', '-c'] + [' '.join(samtools_args)]
        subprocess.call(bash_call,
                        stdout=err_handle,
                        stderr=subprocess.STDOUT)

    err_handle.close()


def bedtools(function, infiles, outfile=None, flags=(), err_file='bedtools.stderr',
             version=None,
             bash_sub=False, bash_dir='/'):
    """
    Call bedtools functions window, getfasta
        window arguments: infiles = [bed/bam file, genome gff file]
                          outfile = output bedfile
                          flags = standard flags are ['-l', '#', '-r', '0', '-sw', '-sm', '-bed']: '-l' and '-r' and the
                                  numbers following them specify the left and right side extension of the window from
                                  either edge of the feature. '-l' corresponds to the start site and '-r' corresponds
                                  to the end of the feature when '-sw' is specified, which tells bedtools window to
                                  define left and right based on strand. '-sm' specifies that overlaps will only be
                                  reported when on the same strand. '-bed' specifies bedfile output.
        getfasta arguments: infiles = [genome fasta, bedfile with genome coordinates]
                            outfile = output fasta file
                            flags = standard flags are ['-name', '-s']: '-name' specifies that the output fasta
                                    identifier will be pulled from the name column (column 3) of the bedfile, '-s'
                                    specifies that strandedness will be respected in extracting fasta sequences.
    :param function: str of function name
    :param infiles: list of str for file paths to input files
    :param outfile: str file path to output file if needed
    :param flags: list of additional flags that can be passed to the given function
    :param err_file: bedtools output/error file
    :param version: Enforces bedtools version number
    :param bash_sub: False/cygwin/winbash
    :param bash_dir: Location of bash.exe, if bash_sub not False
    :return:
    """

    # Check access to samtools
    try:
        if not bash_sub:
            version_call = subprocess.check_output(['bedtools', '--version']).decode('utf-8')
        else:
            bash_check = ['%s/bash' % bash_dir, '--login', '-c', 'bedtools --version']
            version_call = subprocess.check_output(bash_check).decode('utf-8')
    except OSError:
        raise RuntimeError('bedtools not found; check if it is installed and in $PATH\n')

    # Check that version is the expected version
    local_version = version_call.split(' ')[1].split('\n')[0]
    assert version == local_version, 'bedtools version incompatibility %s != %s' % (version, local_version)

    # Check that chosen function is supported
    if function not in ['window', 'getfasta']:
        raise Exception('Function %s not currently supported by python bedtools wrapper' % function)

    # Change file formats for shell option
    if bash_sub:
        for n, infile in enumerate(infiles):
            infiles[n] = windowsToBash(infile, bash_sub)

    # Make sure spaces in files are escaped for bash
    for n, infile in enumerate(infiles):
        if ' ' in infile:
            infiles[n] = '"' + infile + '"'

    # Initialize error file
    err_handle = open(err_file, 'w')

    # Assemble arguments list, call function
    bedtools_args = ['bedtools', function]

    if function == 'window':
        # Check for correct input number files, and for file type of first input. Takes 1 bam/bed, 1 gff file
        if len(infiles) != 2:
            raise IndexError('bedtools window requires 2 input files')

        if infiles[0].replace('"', '').split('.')[-1] == 'bed':
            bedtools_args += ['-a', infiles[0]]
        elif infiles[0].replace('"', '').split('.')[-1] == 'bam':
            bedtools_args += ['-abam', infiles[0]]
        else:
            raise NameError('bedtools window takes only .bed or .bam files as input')

        bedtools_args += ['-b', infiles[1]] + list(flags)

        # Initialize output file
        out_handle = open(outfile, 'w')

        # Call bedtools
        if not bash_sub:
            subprocess.call(bedtools_args,
                            stdout=out_handle,
                            stderr=err_handle)
        else:
            bash_call = ['%s/bash' % bash_dir, '--login', '-c'] + [' '.join(bedtools_args)]
            subprocess.call(bash_call,
                            stdout=out_handle,
                            stderr=err_handle)

        out_handle.close()

    elif function == 'getfasta':
        # Check for correct input number files. Takes 1 fasta, 1 bed file
        # Standard flags for extractSeq function: -name -s
        if len(infiles) != 2:
            raise IndexError('bedtools getfasta requires 2 input files')

        bedtools_args += ['-fi', infiles[0], '-bed', infiles[1]] + list(flags)

        # Initialize output file
        if outfile:
            if bash_sub:
                outfile = windowsToBash(outfile, bash_sub)

            outfile = '"' + outfile + '"'
            bedtools_args += ['-fo', outfile]
        else:
            raise IndexError('getfasta requires an outfile')

        # Call bedtools
        if not bash_sub:
            subprocess.call(bedtools_args,
                            stderr=err_handle)
        else:
            bash_call = ['%s/bash' % bash_dir, '--login', '-c'] + [' '.join(bedtools_args)]
            subprocess.call(bash_call,
                            stderr=err_handle)

    else:
        print('%s function not currently supported' % function)

    err_handle.close()



import datetime
from tkinter import *
import tkinter.filedialog
import functools
import os
from collections import OrderedDict
from . import pna_pipeline
import pna_finder


class pna_app:
    def __init__(self, master, bash_path='/', home_path='/', shell_type='cygwin'):
        """
        Presents user with choice of the two PNA Finder functions: Get Sequences and Find Off-Targets.
        Initializes tkinter frame and file search directories.
        (see manual.pdf in package root directory for more info)
        :param master:
        :param bash_path:
        :param home_path:
        """

        self.master = master
        self.frame_welcome = Frame(self.master)
        self.frame_welcome.pack()
        self.bash_path = bash_path
        self.home_path = home_path
        self.shell_type = shell_type

        Label(
            self.frame_welcome,
            text='Welcome to the PNA Finder Toolbox', font=('Arial', 12, 'bold')
        ).grid(columnspan=2, padx=5, pady=5)

        Label(
            self.frame_welcome,
            text='Select toolbox function:', font=('Arial', 11, 'italic')
        ).grid(rowspan=3, row=1, column=0, padx=5, pady=5)

        self.function = IntVar()
        self.function.set(0)

        Radiobutton(
            self.frame_welcome,
            text='Get Sequences',
            variable=self.function, value=0
        ).grid(row=1, column=1, sticky=W, padx=5, pady=0)

        Radiobutton(
            self.frame_welcome,
            text='Find Off-Targets',
            variable=self.function, value=1
        ).grid(row=2, column=1, sticky=W, padx=5, pady=0)

        Radiobutton(
            self.frame_welcome,
            text='Check Sequence Warnings',
            variable=self.function, value=2
        ).grid(row=3, column=1, sticky=W, padx=5, pady=0)

        Button(
            self.frame_welcome,
            text="OK", command=self.run_pnaFinder
        ).grid(row=10, column=0, sticky=W, padx=5, pady=10)

        Button(
            self.frame_welcome,
            text="QUIT", fg="red", command=self.frame_welcome.quit,
        ).grid(row=10, column=1, sticky=E, padx=5, pady=10)

    def run_pnaFinder(self):
        """
        Initializes dictionaries for PNA Finder GUI labels, parameters, and filenames. Runs the PNA Finder toolbox
        function that the user has selected.
        :return:
        """

        top = self.top = Toplevel(self.master)
        self.master.withdraw()

        self.labels = {}
        self.flabels = {}
        self.parameters = {}
        self.files = {}

        # JOB NAME BOX
        self.labels['job_name'] = Label(
            self.top,
            text='Job Name:', font=('Arial', 11, 'bold'))
        self.labels['job_name'].grid(row=0, column=0, sticky=W, padx=5, pady=5)

        self.job_name = StringVar()
        self.job_name.set('')
        Entry(self.top, width=20, textvariable=self.job_name).grid(row=0, columnspan=3, padx=20, pady=5)

        if self.function.get() == 0:
            top.title('PNA Finder - Get Sequences')
            self.function_name = 'getSequences'
            self.app_gs()
        elif self.function.get() == 1:
            top.title('PNA Finder - Find Off-Targets')
            self.function_name = 'findOffTargets'
            self.app_ot()
        elif self.function.get() == 2:
            top.title('PNA Finder - Check Sequence Warnings')
            self.function_name = 'sequenceWarnings'
            self.app_warnings()

    def app_gs(self):
        """
        Creates a dialog box that may be filled in with the Get Sequences function parameters and file selections
            getSequences Inputs:
                job parameter keys
                    start       -   int
                    end         -   int
                    pna_length  -   int
                    feature_type -  str
                file keys
                    gff_option  -   int
                    id_list     -   str
                    assembly    -   str
                    gff         -   str
                    output      -   str
                option keys
                    STRING      -   int (optional)
                    string_id   -   int (optional)
                    warnings    -   int (optional)
        (see manual.pdf in package root directory for more info)
        :return:
        """

        # PNA SEQUENCE PARAMETERS
        self.labels['start'] = Label(  # Prompt for window start index
            self.top,
            text='Sequence Window Start:')
        self.labels['start'].grid(row=1, column=0, sticky=W, padx=5, pady=5)

        self.labels['end'] = Label(  # Prompt for window end index
            self.top,
            text='Sequence Window End:')
        self.labels['end'].grid(row=2, column=0, sticky=W, padx=5, pady=5)

        self.labels['length'] = Label(  # Prompt for PNA length
            self.top,
            text='PNA Length:')
        self.labels['length'].grid(row=3, column=0, sticky=W, padx=5, pady=5)

        self.labels['feature_type'] = Label(  # Prompt for PNA length
            self.top,
            text='Annotation Record Types:')
        self.labels['feature_type'].grid(row=4, column=0, sticky=W, padx=5, pady=5)

        self.parameters['start'] = StringVar()
        self.parameters['start'].set('-5')

        self.window_start = Entry(self.top, width=5, textvariable=self.parameters['start'])
        self.window_start.grid(row=1, column=2, sticky=E, padx=5, pady=5)

        self.parameters['end'] = StringVar()
        self.parameters['end'].set('-5')

        self.window_end = Entry(self.top, width=5, textvariable=self.parameters['end'])
        self.window_end.grid(row=2, column=2, sticky=E, padx=5, pady=5)

        self.parameters['length'] = StringVar()
        self.parameters['length'].set('12')

        self.pna_length = Entry(self.top, width=5, textvariable=self.parameters['length'])
        self.pna_length.grid(row=3, column=2, sticky=E, padx=5, pady=5)

        self.parameters['feature_type'] = StringVar()
        self.parameters['feature_type'].set('CDS')

        self.pna_length = Entry(self.top, width=10, textvariable=self.parameters['feature_type'])
        self.pna_length.grid(row=4, column=2, sticky=E, padx=5, pady=5)

        # ID LIST FILE SELECTION
        self.labels['id_list'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select Gene ID List'
        )
        self.labels['id_list'].grid(row=7, column=0, sticky=W, padx=5, pady=5)

        id_list_button = Button(  # Button to select gene ID list file
            self.top,
            text="Select",
            command=functools.partial(self.browse, self.top, 'id_list',
                                      title='Select Gene ID List',
                                      filetypes=(("text files", "*.txt"), ("all files", "*.*")),
                                      row=7, column=1))
        id_list_button.grid(row=7, column=2, sticky=E, padx=5, pady=5)

        # GENOME ASSEMBLY FILE SELECTION
        self.labels['assembly'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select Genome Assembly File'
        )
        self.labels['assembly'].grid(row=8, column=0, sticky=W, padx=5, pady=5)

        assembly_button = Button(  # Button to select genome assembly file
            self.top,
            text="Select",
            command=functools.partial(self.browse, self.top, 'assembly',
                                      title='Select genome assembly FASTA File',
                                      filetypes=(("fasta files", "*.fa *.fna"), ("all files", "*.*")),
                                      row=8, column=1))
        assembly_button.grid(row=8, column=2, sticky=E, padx=5, pady=5)

        # ANNOTATION FILE SELECTION
        Label(
            self.top,
            text='Select Annotation File Option:', font=("Arial", 11, "italic")
        ).grid(row=9, rowspan=2, sticky=N, pady=(20, 0))

        self.gff_option = IntVar()
        self.gff_option.set(0)

        Radiobutton(
            self.top,
            text='Build new gffutils database from GFF file',
            variable=self.gff_option, value=0
        ).grid(row=9, column=1, columnspan=2, sticky=W, padx=5, pady=(10, 0))

        Radiobutton(
            self.top,
            text='Select existing gffutils database file',
            variable=self.gff_option, value=1
        ).grid(row=10, column=1, columnspan=2, sticky=W, padx=5, pady=0)

        self.labels['gff'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select Annotation File'
        )
        self.labels['gff'].grid(row=11, column=0, sticky=W, padx=5, pady=5)

        gff_button = Button(
            self.top,
            text="Select",
            command=functools.partial(self.browse, self.top, 'gff',
                                      title='Select Annotation File',
                                      row=11, column=1))
        gff_button.grid(row=11, column=2, sticky=E, padx=5, pady=5)

        # OUTPUT DIRECTORY SELECTION
        self.labels['output'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select Output Directory'
        )
        self.labels['output'].grid(row=12, column=0, sticky=W, padx=5, pady=5)

        output_button = Button(
            self.top,
            text="Select",
            command=functools.partial(self.browse_folder, self.top, 'output',
                                      title='Select Output Directory',
                                      row=12, column=1))
        output_button.grid(row=12, column=2, sticky=E, padx=5, pady=5)

        # STRING ANALYSIS, MELT TEMPERATURE, AND SOLUBILITY WARNINGS OPTIONS
        self.STRING = IntVar()
        self.STRING.set(0)

        self.parameters['string_id'] = StringVar()
        self.parameters['string_id'].set('')

        self.warnings = IntVar()
        self.warnings.set(1)

        self.temp_option = IntVar()
        self.temp_option.set(1)

        self.labels['string_id'] = Label(
            self.top,
            text='Species NCBI Taxonomy ID', font=("Arial", 8, "bold"))
        self.labels['string_id'].grid(row=13, column=1, columnspan=2, sticky=E, pady=(10, 0))

        Checkbutton(
            self.top,
            text='Run STRING analysis',
            variable=self.STRING
        ).grid(row=14, column=0, sticky=W, padx=5, pady=(0, 5))

        self.string_id = Entry(self.top, width=5, textvariable=self.parameters['string_id'])
        self.string_id.grid(row=14, column=2, sticky=E, padx=5, pady=0)

        Checkbutton(
            self.top,
            text='Run sequence warnings analysis',
            variable=self.warnings
        ).grid(row=15, column=0, sticky=W, padx=5, pady=5)

        Checkbutton(
            self.top,
            text='Calculate single-stranded PNA/DNA melt temperature',
            variable=self.temp_option
        ).grid(row=16, column=0, columnspan=3, sticky=W, padx=5, pady=5)

        # JOB SUBMISSION
        Button(
            self.top,
            text="SUBMIT", command=self.submit
        ).grid(row=17, column=0, sticky=W, padx=5, pady=10)

        # WINDOW QUIT
        Button(
            self.top,
            text="QUIT", fg="red", command=self.quit_app,
        ).grid(row=17, column=2, sticky=E, padx=5, pady=10)

    def app_ot(self):
        """
        Creates a dialog box that may be filled in with the Find Off Targets function parameters and file selections
            findOffTargets Inputs:
                job parameter keys
                    start       -   int
                    end         -   int
                    mismatch    -   int
                    length      -   int
                    feature_type -  str
                file keys
                    index_option -  int
                    targets     -   str
                    assembly    -   str
                    gff         -   str
                    output      -   str
                option keys
                    count       -   int (optional)
                    homology    -   int (optional)
                    remove_files -  int (optional)
        (see manual.pdf in package root directory for more info)
        :return:
        """

        # OFF TARGET SEARCH PARAMETERS
        self.labels['start'] = Label(  # Prompt for window start index
            self.top,
            text='Off-Target Window Start:')
        self.labels['start'].grid(row=1, column=0, sticky=W, padx=5, pady=5)

        self.labels['end'] = Label(  # Prompt for window end index
            self.top,
            text='Off-Target Window End:')
        self.labels['end'].grid(row=2, column=0, sticky=W, padx=5, pady=5)

        self.labels['mismatch'] = Label(  # Prompt for number of mismatches allowed
            self.top,
            text='Mismatches:')
        self.labels['mismatch'].grid(row=3, column=0, sticky=W, padx=5, pady=5)

        self.labels['feature_type'] = Label(  # Prompt for GFF record type
            self.top,
            text='Annotation Record Types:')
        self.labels['feature_type'].grid(row=5, column=0, sticky=W, padx=5, pady=5)

        self.parameters['start'] = StringVar()
        self.parameters['start'].set('-20')

        self.window_start = Entry(self.top, width=5, textvariable=self.parameters['start'])
        self.window_start.grid(row=1, column=2, sticky=E, padx=5, pady=5)

        self.parameters['end'] = StringVar()
        self.parameters['end'].set('20')

        self.window_end = Entry(self.top, width=5, textvariable=self.parameters['end'])
        self.window_end.grid(row=2, column=2, sticky=E, padx=5, pady=5)

        self.parameters['mismatch'] = StringVar()
        self.parameters['mismatch'].set('0')

        self.mismatch = Entry(self.top, width=5, textvariable=self.parameters['mismatch'])
        self.mismatch.grid(row=3, column=2, sticky=E, padx=5, pady=5)

        self.parameters['feature_type'] = StringVar()
        self.parameters['feature_type'].set('CDS')

        self.feature_type = Entry(self.top, width=10, textvariable=self.parameters['feature_type'])
        self.feature_type.grid(row=5, column=2, sticky=E, padx=5, pady=5)

        # PNA FASTA FILE SELECTION
        self.labels['targets'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select PNA Targets File'
        )
        self.labels['targets'].grid(row=6, column=0, sticky=W, padx=5, pady=5)

        targets_button = Button(
            self.top,
            text="Select",
            command=functools.partial(self.browse, self.top, 'targets',
                                      title='Select PNA targets FASTA file',
                                      filetypes=(("fasta files", "*.fa *.fna"), ("all files", "*.*")),
                                      row=6, column=1))
        targets_button.grid(row=6, column=2, sticky=E, padx=5, pady=5)

        # GENOME ASSEMBLY FILE SELECTION
        Label(
            self.top,
            text='Select Index File Option:', font=("Arial", 11, "italic")
        ).grid(row=7, rowspan=2, sticky=N, pady=(20, 0))

        self.index_option = IntVar()
        self.index_option.set(0)

        Radiobutton(
            self.top,
            text='Build new index from FASTA file',
            variable=self.index_option, value=0
        ).grid(row=8, column=1, columnspan=2, sticky=W, padx=5, pady=(5, 0))

        Radiobutton(
            self.top,
            text='Select existing index (any of the 6 files)',
            variable=self.index_option, value=1
        ).grid(row=9, column=1, columnspan=2, sticky=W, padx=5, pady=(0, 5))

        self.labels['assembly'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select Assembly/Index File'
        )
        self.labels['assembly'].grid(row=10, column=0, sticky=W, padx=5, pady=5)

        assembly_button = Button(
            self.top,
            text="Select",
            command=functools.partial(self.browse, self.top, 'assembly',
                                      title='Select Assembly/Index File',
                                      row=10, column=1))
        assembly_button.grid(row=10, column=2, sticky=E, padx=5, pady=5)

        # ANNOTATION FILE SELECTION
        self.labels['gff'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select Annotation File'
        )
        self.labels['gff'].grid(row=11, column=0, sticky=W, padx=5, pady=5)

        gff_button = Button(
            self.top,
            text="Select",
            command=functools.partial(self.browse, self.top, 'gff',
                                      title='Select Annotation File',
                                      filetypes=(("gff files", "*.gff *.gtf"), ("all files", "*.*")),
                                      row=11, column=1))
        gff_button.grid(row=11, column=2, sticky=E, padx=5, pady=5)

        # OUTPUT DIRECTORY SELECTION
        self.labels['output'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select Output Directory'
        )
        self.labels['output'].grid(row=12, column=0, sticky=W, padx=5, pady=5)

        output_button = Button(
            self.top,
            text="Select",
            command=functools.partial(self.browse_folder, self.top, 'output',
                                      title='Select Output Directory',
                                      row=12, column=1))
        output_button.grid(row=12, column=2, sticky=E, padx=5, pady=5)

        # FILE OUTPUT OPTIONS
        self.count = IntVar()
        self.count.set(1)

        self.homology = IntVar()
        self.homology.set(0)

        self.remove_files = IntVar()
        self.remove_files.set(0)

        Checkbutton(
            self.top,
            text='Create off-target count file',
            variable=self.count
        ).grid(row=13, column=0, sticky=W, padx=5, pady=5)

        Checkbutton(
            self.top,
            text='Check for potential feature homology',
            variable=self.homology
        ).grid(row=14, column=0, columnspan=2, sticky=W, padx=5, pady=5)

        Checkbutton(
            self.top,
            text='Remove intermediate and error files after run',
            variable=self.remove_files
        ).grid(row=15, column=0, columnspan=2, sticky=W, padx=5, pady=5)

        # JOB SUBMISSION
        Button(
            self.top,
            text="SUBMIT", command=self.submit
        ).grid(row=16, column=0, sticky=W, padx=5, pady=10)

        # WINDOW QUIT
        Button(
            self.top,
            text="QUIT", fg="red", command=self.quit_app,
        ).grid(row=16, column=2, sticky=E, padx=5, pady=10)

    def app_warnings(self):
        """
        Creates a dialog box that may be filled in with the Check Warnings file selections
            sequenceWarnings Inputs:
                self.files dictionary
                    pna_list    -   str
                    output      -   str
        (see manual.pdf in package root directory for more info)
        :return:
        :return:
        """

        # PNA FILE SELECTION
        Label(
            self.top,
            text='Select PNA Upload Filetype:', font=("Arial", 11, "italic")
        ).grid(row=1, rowspan=2, sticky=N, pady=(20, 0))

        self.pna_option = IntVar()
        self.pna_option.set(0)

        Radiobutton(
            self.top,
            text='Upload PNA target FASTA file',
            variable=self.pna_option, value=0
        ).grid(row=1, column=1, columnspan=2, sticky=W, padx=5, pady=(10, 0))

        Radiobutton(
            self.top,
            text='Upload PNA sequences table',
            variable=self.pna_option, value=1
        ).grid(row=2, column=1, columnspan=2, sticky=W, padx=5, pady=0)

        self.labels['pna_list'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select PNA File'
        )
        self.labels['pna_list'].grid(row=3, column=0, sticky=W, padx=5, pady=5)

        pna_button = Button(
            self.top,
            text="Select",
            command=functools.partial(self.browse, self.top, 'pna_list',
                                      title='Select PNA File',
                                      row=3, column=1))
        pna_button.grid(row=3, column=2, sticky=E, padx=5, pady=5)

        # OUTPUT DIRECTORY SELECTION
        self.labels['output'] = Label(
            self.top,
            font=('Arial', 9, 'bold'),
            text='Select Output Directory'
        )
        self.labels['output'].grid(row=4, column=0, sticky=W, padx=5, pady=5)

        output_button = Button(
            self.top,
            text="Select",
            command=functools.partial(self.browse_folder, self.top, 'output',
                                      title='Select Output Directory',
                                      row=4, column=1))
        output_button.grid(row=4, column=2, sticky=E, padx=5, pady=5)

        # MELT TEMPERATURE CALCULATION OPTION
        self.temp_option = IntVar()
        self.temp_option.set(1)

        Checkbutton(
            self.top,
            text='Calculate single-stranded PNA/DNA melt temperature',
            variable=self.temp_option
        ).grid(row=5, column=0, columnspan=3, sticky=W, padx=5, pady=5)

        # JOB SUBMISSION
        Button(
            self.top,
            text="SUBMIT", command=self.submit
        ).grid(row=6, column=0, sticky=W, padx=5, pady=10)

        # WINDOW QUIT
        Button(
            self.top,
            text="QUIT", fg="red", command=self.quit_app,
        ).grid(row=6, column=2, sticky=E, padx=5, pady=10)

    def browse(self, frame, filekey, title='Select file', filetypes=(("all files", "*.*"),),
               row=0, column=0):
        """
        Function prompts user to select a file for input into the PNA Finder toolbox, and records choice in GUI display
        :param frame: Tkinter active frame
        :param filekey: Key for the self.files dictionary, corresponding to the file being selected
        :param title: File browser window title
        :param filetypes: File browser possible filetype selections
        :param row: Row location of file label within the Tkinter frame
        :param column: Column location of file label within the Tkinter frame
        :return:
        """

        try:
            if self.flabels['%s_display' % filekey].get():
                label_pass = True
            else:
                label_pass = False
        except KeyError:
            self.flabels['%s_display' % filekey] = StringVar()
            label_pass = False

        if filekey == 'gff' and self.function.get() == 0:
            if self.gff_option.get() == 0:
                filetypes = (("gff/gtf files", "*.gff *gtf"), ("all files", "*.*"))
            elif self.gff_option.get() == 1:
                filetypes = (("db files", "*.db"), ("all files", "*.*"))
            else:
                raise ValueError('Something has gone horribly wrong!')

        elif filekey == 'assembly' and self.function.get() == 1:
            if not self.index_option.get():
                filetypes = (("fasta files", "*.fa *.fna"), ("all files", "*.*"))
                title = 'Select genome assembly FASTA file'
            elif self.index_option.get():
                filetypes = (("Bowtie index files", "*.ebwt"), ("all files", "*.*"))
                title = 'Select genome assembly BT index file'
            else:
                raise ValueError('Something has gone horribly wrong!')

        elif filekey == 'pna_list':
            if not self.pna_option.get():
                filetypes = (("fasta files", "*.fa *.fna"), ("all files", "*.*"))
                title = 'Select PNA target FASTA file'
            if self.pna_option.get():
                filetypes = (("text files", "*.txt"), ("all files", "*.*"))
                title = 'Select PNA sequence table file'

        f_out = str(tkinter.filedialog.askopenfilename(initialdir=self.home_path, title=title,
                                                       filetypes=filetypes))  # TODO: try askopenfilenames

        if f_out:
            self.files[filekey] = f_out
            file_loc_list = self.files[filekey].split('/')
            file_type = file_loc_list[-1].split('.')[-1]

            if len(file_loc_list[-1]) > 15:
                if len(file_loc_list) > 2:
                    if len(file_loc_list[-1].rsplit('.', 1)[0]) > 15:
                        temp_label = file_loc_list[0] + '/.../' + file_loc_list[-1].rsplit('.', 1)[0][
                                                                  0:15] + '... .' + file_type
                    else:
                        temp_label = file_loc_list[0] + '/.../' + file_loc_list[-1]
                    self.flabels['%s_display' % filekey].set(temp_label)
                else:
                    temp_label = file_loc_list[0] + '/' + file_loc_list[-1][0:15] + '... .' + file_type
                    self.flabels['%s_display' % filekey].set(temp_label)
            else:
                if len(file_loc_list) > 2:
                    temp_label = file_loc_list[0] + '/.../' + file_loc_list[-1]
                    self.flabels['%s_display' % filekey].set(temp_label)
                else:
                    self.flabels['%s_display' % filekey].set(self.files[filekey])

            if label_pass:
                pass
            else:
                Label(
                    frame,
                    textvariable=self.flabels['%s_display' % filekey]
                ).grid(row=row, column=column, sticky=E, padx=5, pady=5)

        return

    def browse_folder(self, frame, filekey, title='Select folder', row=0, column=0):
        """
        Function prompts user to select a folder for input into the PNA Finder toolbox, and records choice in GUI display
        :param frame: Tkinter active frame
        :param filekey: Key for the self.files dictionary, corresponding to the folder being selected
        :param title: File browser window title
        :param row: Row location of file label within the Tkinter frame
        :param column: Column location of file label within the Tkinter frame
        :return:
        """

        try:
            if self.flabels['%s_display' % filekey]:
                label_pass = True
            else:
                label_pass = False
        except KeyError:
            self.flabels['%s_display' % filekey] = StringVar()
            label_pass = False

        d_out = str(tkinter.filedialog.askdirectory(initialdir=self.home_path, title=title))

        if d_out:
            self.files[filekey] = d_out
            file_loc_list = self.files[filekey].split('/')

            if len(file_loc_list[-1]) > 15:
                if len(file_loc_list) > 2:
                    self.flabels['%s_display' % filekey].set(file_loc_list[0] + '/.../' + file_loc_list[-1][0:15])
                else:
                    self.flabels['%s_display' % filekey].set(file_loc_list[0] + '/' + file_loc_list[-1][0:15])
            else:
                if len(file_loc_list) > 2:
                    self.flabels['%s_display' % filekey].set(file_loc_list[0] + '/.../' + file_loc_list[-1])
                else:
                    self.flabels['%s_display' % filekey].set(self.files[filekey])

            if label_pass:
                pass
            else:
                Label(
                    frame,
                    textvariable=self.flabels['%s_display' % filekey]
                ).grid(row=row, column=column, sticky=E, padx=5, pady=5)

        return

    def check_inputs(self):
        """
        Function checks the inputs for PNA tools to determine whether the necessary inputs are submitted in the
        correct format
            getSequences Inputs:
                self.parameters dictionary:
                    start       -   int
                    end         -   int
                    pna_length  -   int
                    feature_type -   str
                    string_id   -   int (optional)
                self.files dictionary
                    id_list     -   str
                    assembly    -   str
                    gff         -   str
                    output      -   str
                self.STRING     -   int (optional)
                self.warnings   -   int (optional)

            findOffTargets Inputs:
                self.parameters dictionary:
                    start       -   int
                    end         -   int
                    mismatch    -   int
                    feature_type -  str
                self.files dictionary
                    targets     -   str
                    assembly    -   str
                    gff         -   str
                    output      -   str
                self.count      -   int (optional)
                self.homology   -   int (optional)
                self.remove_files - int (optional)

            sequenceWarnings Inputs
                pna_option  -   int
                pna_list    -   str
                temp_option -   int (optional)
        """

        check = True
        check_warnings = OrderedDict()
        check_messages = OrderedDict()
        submit_dict = OrderedDict()
        p_dict = f_dict = None

        if self.job_name.get() == '':
            now = datetime.datetime.now()
            self.job_name.set('%s_%s' % (str(now.date()), self.function_name))
        else:
            for character in '<>:"/\|?*':
                if character in self.job_name.get():
                    check = False
                    check_warnings['job_name'] = r'Job name must be valid directory name (Remove characters: <>:"/\|?*)'

        if self.function.get() == 0:
            p_dict = OrderedDict([('start', 'Sequence Window Start'),
                                  ('end', 'Sequence Window End'),
                                  ('length', 'PNA Length'),
                                  ('feature_type', 'Annotation Record Types'),
                                  ('string_id', 'STRING Species ID')])

            f_dict = OrderedDict([('id_list', 'Gene ID List'),
                                  ('assembly', 'Genome Assembly File'),
                                  ('gff', 'Annotation File'),
                                  ('output', 'Output Directory')])

            submit_dict['gff_option'] = self.gff_option.get()
            submit_dict['STRING'] = self.STRING.get()
            submit_dict['warnings'] = self.warnings.get()
            submit_dict['temp_option'] = self.temp_option.get()

        elif self.function.get() == 1:
            p_dict = OrderedDict([('start', 'Off-Target Window Start'),
                                  ('end', 'Off-Target Window End'),
                                  ('mismatch', 'Mismatches'),
                                  ('feature_type', 'Annotation Record Types')])

            f_dict = OrderedDict([('targets', 'PNA Targets File'),
                                  ('assembly', 'Assembly/Index File'),
                                  ('gff', 'Annotation File'),
                                  ('output', 'Output Directory')])

            submit_dict['index_option'] = self.index_option.get()
            submit_dict['count'] = self.count.get()
            submit_dict['homology'] = self.homology.get()
            submit_dict['remove_files'] = self.remove_files.get()

        elif self.function.get() == 2:
            p_dict = OrderedDict([])
            f_dict = OrderedDict([('pna_list', 'PNA Sequences File'),
                                  ('output', 'Output Directory')])

            submit_dict['pna_option'] = self.pna_option.get()
            submit_dict['temp_option'] = self.temp_option.get()

        for p_key in p_dict:
            try:  # Entry data check
                submit_dict[p_key] = int(self.parameters[p_key].get().strip())
                if self.function.get() == 0 and p_key == 'string_id' and not self.STRING.get():
                    check_messages[p_key] = 'STRING ID entered but STRING analysis option not selected'

                elif p_key == 'end' and submit_dict[p_key] < submit_dict['start']:
                    check = False
                    check_warnings[p_key] = 'Window Start must be less than or equal to Window End'

            except ValueError:
                if self.function.get() == 0:
                    if p_key == 'string_id' and not self.STRING.get():
                        continue
                elif self.function.get() == 1:
                    if p_key == 'end' and self.parameters[p_key].get() == '':
                        submit_dict[p_key] = self.parameters[p_key].get()
                        continue
                    elif p_key == 'mismatch':
                        try:
                            mismatch_list = []
                            for mismatch in self.parameters[p_key].get().split(','):
                                mismatch_list += [int(mismatch.strip())]
                            submit_dict[p_key] = mismatch_list
                            continue
                        except ValueError:
                            check = False
                            check_warnings[p_key] = 'Mismatch must be integer or comma-separated list of integers'
                            continue

                if p_key == 'feature_type':
                    submit_dict[p_key] = self.parameters[p_key].get().strip()
                    continue

                check = False
                if self.parameters[p_key].get() == '':
                    check_warnings[p_key] = 'No value entered for %s' % p_dict[p_key]
                else:
                    check_warnings[p_key] = 'Must enter integer value for %s' % p_dict[p_key]

        for f_key in f_dict:
            try:
                if os.path.exists(self.files[f_key]):
                    if f_key == 'output':
                        submit_dict[f_key] = str(self.files[f_key].rstrip()) + '/' + str(self.job_name.get().rstrip())

                    elif self.function.get() == 0 and f_key == 'gff' and \
                            ((self.gff_option.get() and self.files[f_key].rsplit('.', 1)[1] != 'db') or
                             (not self.gff_option.get() and self.files[f_key].rsplit('.', 1)[1] != 'gff')):
                        check = False
                        check_warnings[f_key] = 'Wrong filetype selected for Annotation File Option'

                    elif self.function.get() and f_key == 'assembly' and \
                            ((self.index_option.get() and self.files[f_key].rsplit('.', 1)[1] != 'ebwt' or
                              (not self.index_option.get() and self.files[f_key].rsplit('.', 1)[1] not in ['fa', 'fna',
                                                                                                           'fasta']))):
                        check = False
                        check_warnings[f_key] = 'Wrong filetype selected for Index File Option'

                    else:
                        submit_dict[f_key] = str(self.files[f_key].rstrip())

                else:
                    check = False
                    check_warnings[f_key] = 'Filepath for %s does not exist' % f_dict[f_key]

            except KeyError:
                check = False
                check_warnings[f_key] = ('Must select %s' % f_dict[f_key])

        return check, submit_dict, check_warnings, check_messages

    def submit(self):
        """
        Calls the self.check_inputs() function and displays warning/error messages if input criteria are not satisfied.
        If input criteria are satisfied, the function submits the job to the external pna_pipeline.py script according
        to the variable self.function.
        :return:
        """

        check, submit_dict, check_warnings, check_messages = self.check_inputs()

        for l_key in self.labels:
            self.labels[l_key].config(fg='black')

        if not check:
            try:
                self.submit_warning.destroy()
            except AttributeError:
                pass

            warning_text = ''

            for w_key in check_warnings:
                self.labels[w_key].config(fg='red')
                warning_text += check_warnings[w_key] + '\n'

            self.submit_warning = Label(
                self.top,
                text=warning_text, fg='red')
            self.submit_warning.grid(row=20, columnspan=3, padx=5, pady=5)
            return

        else:
            submit_dict['home'] = self.home_path
            submit_dict['bash'] = self.bash_path
            submit_dict['shell_type'] = self.shell_type

            try:
                self.submit_warning.destroy()
            except AttributeError:
                pass

            if check_messages:
                message_text = ''

                for m_key in check_messages:
                    self.labels[m_key].config(fg='#FF6D00')
                    message_text += check_messages[m_key] + '\n'

                self.submit_warning = Label(
                    self.top,
                    text=message_text, fg='#FF6D00')
                self.submit_warning.grid(row=20, column=0, columnspan=3, padx=5, pady=5)

            start_time = datetime.datetime.now()
            print('Started at %s' % start_time)

            if self.function.get() == 0:
                pna_pipeline_bt.getSequences(submit_dict)
            elif self.function.get() == 1:
                pna_pipeline_bt.findOffTargets(submit_dict)
            elif self.function.get() == 2:
                pna_pipeline_bt.sequenceWarnings(submit_dict)

            end_time = datetime.datetime.now()
            print('Finished at %s' % end_time)
            print('Run execution time: %s' % (end_time - start_time))

    def quit_app(self):
        self.top.destroy()
        self.master.deiconify()


def startup():
    """
    Checks for proper start_info.txt file, to ensure preset home and bash directories.
    (see manual.pdf in package root directory for more info)
    :return:
    """

    bash_path = home_path = shell_type = False
    warning_text = 'The file "start_info.txt" has been deleted or moved.'

    try:
        start_handle = open('%s/start_info.txt' % pna_finder_dev.__path__[0], 'r')
        warning_text = 'The file "start_info.txt" is improperly built.'

        start_dict = {}
        for line in start_handle:
            path_dict = line.rstrip().split('\t')
            start_dict[path_dict[0]] = path_dict[1]

        home_path = start_dict['home']
        bash_path = start_dict['bash']
        shell_type = start_dict['shell']

    except IOError:
        start_warning = Tk()
        start_warning.title('PNA Finder Toolbox')

        Label(
            start_warning,
            text='ERROR', fg='red',
            font=('Arial', 12, 'bold')
        ).grid(row=0, padx=5, pady=5)

        Label(
            start_warning,
            text='%s\nRun start_info.py to rebuild this file.' % warning_text,
            font=('Arial', 10, 'italic')
        ).grid(row=1, padx=5, pady=5)

        Button(
            start_warning,
            text="QUIT", fg='red', command=start_warning.quit
        ).grid(row=2, padx=5, pady=10)

        start_warning.mainloop()
        start_warning.destroy()

    return bash_path, home_path, shell_type


def run():
    """
    Runs the PNA Finder toolbox GUI
    :return:
    """
    bash_path, home_path, shell_type = startup()

    if bash_path and home_path:
        root = Tk()
        root.title('PNA Finder Toolbox')

        pna_app(root, bash_path=bash_path, home_path=home_path, shell_type=shell_type)

        root.mainloop()
        root.destroy()



from setuptools import setup, find_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pna-finder',
    version='0.2',
    author='Thomas Aunins',
    author_email='',
    description='A toolbox for identifying PNA antisense sequences',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/taunins/pna_finder',
    license='GNU LGPLv3',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: Microsoft',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'
    ],
    python_requires='>=2.7',
    install_requires=[
        'gffutils',
        'biopython'
    ]

)
